use serde::ser::{self, Serializer};
use thiserror::Error;

pub mod profile;

#[derive(Error, Debug)]
pub enum Error {
    #[error("Issue at the serializer: {0}")]
    Json(#[from] serde_json::Error),
}

#[derive(Eq, PartialEq, Clone, Debug)]
pub enum Content {
    Profile(profile::Profile),
    Text(String),
    RecommendServer(String),
    Unknown(u32, String),
}

impl ser::Serialize for Content {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(
            &self
                .try_to_string()
                .map_err(|e| ser::Error::custom(e.to_string()))?,
        )
    }
}

impl Content {
    pub fn kind(&self) -> u32 {
        match self {
            Self::Profile(_) => 0,
            Self::Text(_) => 1,
            Self::RecommendServer(_) => 2,
            Self::Unknown(k, _) => *k,
        }
    }

    pub fn deserialize(kind: u32, content: &str) -> Result<Self, Error> {
        match kind {
            0 => Ok(Self::Profile(serde_json::from_str(content)?)),
            1 => Ok(Self::Text(content.to_owned())),
            2 => Ok(Self::RecommendServer(content.to_owned())),
            _ => Ok(Self::Unknown(kind, content.to_owned())),
        }
    }

    pub fn try_to_string(&self) -> Result<String, Error> {
        match self {
            Self::Profile(p) => Ok(serde_json::to_string(&p)?),
            Self::Text(t) | Self::RecommendServer(t) | Self::Unknown(_, t) => Ok(t.to_owned()),
        }
    }
}
